from django.conf import settings as django_settings


_settings = {
    'MAX_COUNT': 2,
    'PROCESS_BOUNCE': True,
    'PROCESS_COMPLAINT': True,
}


class Settings(object):
    def __getattribute__(self, name):
        sdict = getattr(django_settings, 'SES_FEEDBACK', {})
        return sdict.get(name, _settings.get(name))


settings = Settings()



# Generated by Django 2.2.9 on 2019-12-28 07:59

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.utils.timezone
import django_ses_feedback.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='EmailAddressNotification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(default=django.utils.timezone.now)),
                ('modified', models.DateTimeField(default=django.utils.timezone.now)),
                ('email', models.EmailField(max_length=255)),
                ('count', models.PositiveIntegerField(default=0)),
                ('notifications', django.contrib.postgres.fields.jsonb.JSONField(default=django_ses_feedback.models._default_notification_dict)),
            ],
        ),
    ]



# Generated by Django 3.2 on 2021-05-10 19:17

from django.db import migrations, models
import django_ses_feedback.models


class Migration(migrations.Migration):

    dependencies = [
        ('django_ses_feedback', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailaddressnotification',
            name='notifications',
            field=models.JSONField(default=django_ses_feedback.models._default_notification_dict),
        ),
    ]



from django.dispatch import receiver

from django_ses.signals import bounce_received, complaint_received

from .settings import settings
from .models import EmailAddressNotification


def _process_notification(notification_type, obj):
    assert(notification_type in ['bounce', 'complaint'])
    if notification_type == 'bounce':
        obj_var = 'bouncedRecipients'
    else:
        obj_var = 'complainedRecipients'

    for bnc in obj[notification_type][obj_var]:
        _email = bnc['emailAddress'].lower()
        try:
            _en = EmailAddressNotification.objects.get(
                email__iexact=_email,
            )
        except EmailAddressNotification.DoesNotExist:
            _en = EmailAddressNotification(email=_email)
        getattr(_en, f'add_{notification_type}')(obj)  # .save() called here


@receiver(
    bounce_received,
    dispatch_uid='django_ses_feedback.signals.process_bounce',
)
def process_bounce(sender, **kwargs):
    if not settings.PROCESS_BOUNCE:
        return

    _process_notification(
        'bounce',
        {'mail': kwargs['mail_obj'], 'bounce': kwargs['bounce_obj']},
    )


@receiver(
    complaint_received,
    dispatch_uid='django_ses_feedback.signals.process_complaint',
)
def process_complaint(sender, **kwargs):
    if not settings.PROCESS_COMPLAINT:
        return

    _process_notification(
        'complaint',
        {'mail': kwargs['mail_obj'], 'complaint': kwargs['complaint_obj']},
    )



import functools

from django.db.models.query import Q, QuerySet

from .models import EmailAddressNotification


def can_send_email(email_addresses, clean=True):
    """ Function to take a list of email addresses and return a dictionary
        of email addresses and their notification status. This is to avoid
        emailing addresses that have too many notifications (bounces or
        complaints)

        if clean is True then return a list of all email addresses that
        we're allowed to send email to.
    """
    if not email_addresses:
        return email_addresses

    assert hasattr(email_addresses, "__iter__")
    if isinstance(email_addresses, QuerySet):
        email_addresses = list(email_addresses)
    elif isinstance(email_addresses, str):
        email_addresses = [email_addresses]

    email_addresses = [x.lower() for x in email_addresses]

    # Not using defaultdict because we don't want blanket True's for
    # addresses not present in email_addresses
    ok_emails = {
        x: True for x in list(set([x.lower() for x in email_addresses]))
    }
    q_list = map(lambda x: Q(email__iexact=x), ok_emails.keys())
    q_list = functools.reduce(lambda a, b: a | b, q_list)
    notifications = EmailAddressNotification.objects.filter(q_list).distinct()

    for n in notifications:
        if clean and not n.email_ok:
            del ok_emails[n.email.lower()]
        else:
            ok_emails[n.email.lower()] = n.email_ok

    if clean:
        # This is stupid but needed because tests expect
        # same ordering when comparing recipient lists
        # ok_emails = list(ok_emails.keys())  # keys() returns dict_keys object
        ok_emails = sorted(
            ok_emails.keys(), key=lambda x: email_addresses.index(x),
        )

    return ok_emails



import os

from setuptools import find_packages, setup

project_name = 'django_ses_feedback'

if os.path.exists('README.rst'):
    long_description = open('README.rst', 'r').read()
else:
    long_description = (
        'See https://hg.code.netlandish.com/~netlandish/django-ses-feedback/'
    )


setup(
    name=project_name,
    version=__import__(project_name).get_version(),
    packages=find_packages(),
    description='Process Amazon SES Feedback',
    author='Netlandish Inc.',
    author_email='hello@netlandish.com',
    url='https://hg.code.netlandish.com/~netlandish/django-ses-feedback/',
    long_description=long_description,
    platforms=['any'],
    install_requires=['Django>=1.8', 'boto3'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Environment :: Web Environment',
    ],
    include_package_data=True,
)



from django.db import models
from django.utils import timezone

from .settings import settings


def _default_notification_dict():
    return {
        'bounces': [],
        'complaints': [],
    }


class EmailAddressNotification(models.Model):
    """ Model to track email notifications. Bounces and complaints.
    """
    created = models.DateTimeField(default=timezone.now)
    modified = models.DateTimeField(default=timezone.now)
    email = models.EmailField(max_length=255)

    # Track complaints and bounces
    count = models.PositiveIntegerField(default=0)
    notifications = models.JSONField(default=_default_notification_dict)

    def reset_notification_count(self):
        self.count = 0
        self.save()

    @property
    def email_ok(self):
        return self.count < settings.MAX_COUNT

    def add_notification(self, notification_type, data):
        """ notification_type should be 'bounce' or 'complaint'
            data is the notification dict.
            ref: https://docs.aws.amazon.com/ses/latest/DeveloperGuide/notification-examples.html
        """
        assert(notification_type in ['bounce', 'complaint'])
        self.count += 1
        self.notifications[f'{notification_type}s'].append(data)
        self.save()

    def add_bounce(self, data):
        return self.add_notification('bounce', data)

    def add_complaint(self, data):
        return self.add_notification('complaint', data)
